from typing import Any, Callable

from fastapi import HTTPException
from ...models.CRUDLookup import CRUDLookup
from .CRUDLookupRouterFactory import CRUDLookupRouterFactory
from . import CRUDLookupRouterRepository


class CRUDLookupRouter(CRUDLookupRouterFactory):

    def __init__(self, parent_router, child_args: CRUDLookup, *args, **kwargs):
        self.prefix = "/{id}/" + child_args.prefix
        self.db = parent_router.db
        self.model = child_args.model
        self.collection_name = child_args.collection_name
        self.local_field = child_args.local_field
        self.foreign_field = child_args.foreign_field
        super().__init__(parent_router, child_args, *args, **kwargs)
        self._register_routes()

    def _get_all(self, *args: Any, **kwargs: Any) -> Callable[..., Any]:
        async def route(id: str) -> self.parent_router.model:
            response = await CRUDLookupRouterRepository.get_all(self.db, self.collection_name, id, self.foreign_field, self.local_field, self.parent_router.collection_name, self.parent_router.model)
            if (response is None):
                raise HTTPException(404, "Empty collection")
            return response
        return route

    def _get_one(self, *args: Any, **kwargs: Any) -> Callable[..., Any]:
        async def route(id: str, lookup_id: str) -> self.parent_router.model:
            response = await CRUDLookupRouterRepository.get_one(self.db, self.collection_name, id, lookup_id, self.foreign_field, self.local_field, self.parent_router.collection_name, self.parent_router.model)
            if (response is None):
                raise HTTPException(404, "Document not found")
            return response
        return route

    def _create_one(self, *args: Any, **kwargs: Any) -> Callable[..., Any]:
        async def route(id: str, data: self.model) -> self.parent_router.model:
            print(self.model)
            response = await CRUDLookupRouterRepository.create_one(self.db, self.collection_name, id, data, self.foreign_field, self.local_field, self.parent_router.collection_name, self.parent_router.model)
            if (response is None):
                raise HTTPException(422, "Document not created")
            return response
        return route

    def _replace_one(self, *args: Any, **kwargs: Any) -> Callable[..., Any]:
        async def route(id: str, lookup_id: str, data: self.model) -> self.parent_router.model:
            response = await CRUDLookupRouterRepository.replace_one(self.db, self.collection_name, id, lookup_id, data, self.foreign_field, self.local_field, self.parent_router.collection_name, self.parent_router.model)
            if (response is None):
                raise HTTPException(422, "Document not replaced")
            return response
        return route

    def _update_one(self, *args: Any, **kwargs: Any) -> Callable[..., Any]:
        async def route(id: str, lookup_id: str, data: self.model) -> self.parent_router.model:
            response = await CRUDLookupRouterRepository.update_one(self.db, self.collection_name, id, lookup_id, data, self.foreign_field, self.local_field, self.parent_router.collection_name, self.parent_router.model)
            if (response is None):
                raise HTTPException(422, "Document not updated")
            return response
        return route

    def _delete_one(self, *args: Any, **kwargs: Any) -> Callable[..., Any]:
        async def route(id: str, lookup_id: str) -> self.model:
            response = await CRUDLookupRouterRepository.delete_one(self.db, self.collection_name, id, lookup_id, self.foreign_field, self.local_field, self.parent_router.collection_name, self.parent_router.model)
            if (response is None):
                raise HTTPException(422, "Document not deleted")
            return response
        return route

    def _register_routes(self):
        self._add_api_route(
            path=self.prefix,
            endpoint=self._get_all(),
            response_model=self.parent_router.model,
            methods=["GET"],
            summary=f"Get All {self.model.__name__} linked to a {self.parent_router.model.__name__}",
            description=f"Get All {self.model.__name__} linked to a {self.parent_router.model.__name__}"
        )
        self._add_api_route(
            path=self.prefix + "/{lookup_id}",
            endpoint=self._get_one(),
            response_model=self.parent_router.model,
            methods=["GET"],
            summary=f"Get 0ne {self.model.__name__} linked to a {self.parent_router.model.__name__} by lookup_id",
            description=f"Get One {self.model.__name__} linked to a {self.parent_router.model.__name__} by lookup_id"
        )
        self._add_api_route(
            path=self.prefix,
            endpoint=self._create_one(),
            response_model=self.parent_router.model,
            methods=["POST"],
            summary=f"Create One {self.model.__name__} linked to a {self.parent_router.model.__name__}",
            description=f"Create One {self.model.__name__} linked to a {self.parent_router.model.__name__}"
        )
        self._add_api_route(
            path=self.prefix + "/{lookup_id}",
            endpoint=self._replace_one(),
            response_model=self.parent_router.model,
            methods=["PUT"],
            summary=f"Replace One {self.model.__name__} linked to a {self.parent_router.model.__name__} by lookup_id",
            description=f"Replace One {self.model.__name__} linked to a {self.parent_router.model.__name__} by lookup_id"
        )
        self._add_api_route(
            path=self.prefix + "/{lookup_id}",
            endpoint=self._update_one(),
            response_model=self.parent_router.model,
            methods=["PATCH"],
            summary=f"Update One {self.model.__name__} linked to a {self.parent_router.model.__name__} by lookup_id",
            description=f"Update One {self.model.__name__} linked to a {self.parent_router.model.__name__} by lookup_id"
        )
        self._add_api_route(
            path=self.prefix + "/{lookup_id}",
            endpoint=self._delete_one(),
            response_model=self.parent_router.model,
            methods=["DELETE"],
            summary=f"Delete One {self.model.__name__} linked to a {self.parent_router.model.__name__} by lookup_id",
            description=f"Delete One {self.model.__name__} linked to a {self.parent_router.model.__name__} by lookup_id"
        )





from typing import Any
from .mongo_model import MongoModel


class CRUDLookup():
    def __init__(self, model: MongoModel, collection_name: str, prefix: str, local_field: str, foreign_field: str):
        self.model = model
        self.collection_name = collection_name
        self.prefix = prefix
        self.local_field = local_field
        self.foreign_field = foreign_field



from bson import ObjectId
from ...models.mongo_model import MongoModel


async def get_all(db, collection_name: str, id: str, foreign_field: str, local_field: str, parent_collection_name: str, parent_model: MongoModel) -> list:
    """
    Get all documents from the database with a lookup
    """
    documents = db[parent_collection_name].aggregate([
        {
            '$match': {
                '_id': ObjectId(id)
            }
        }, {
            '$lookup': {
                'from': collection_name,
                'localField': local_field,
                'foreignField': foreign_field,
                'as': collection_name
            }
        }
    ])
    models = []
    async for document in documents:
        models.append(parent_model.from_mongo(document))
    return models[0]


async def get_one(db, collection_name: str, id: str, lookup_id: str, foreign_field: str, local_field: str, parent_collection_name: str, parent_model: MongoModel) -> MongoModel:
    """
    Get one document from the database with a lookup
    """
    documents = db[parent_collection_name].aggregate(
        [
            {'$match': {'_id': ObjectId(id)}},
            {
                '$lookup': {
                    'from': collection_name,
                    'localField': local_field,
                    'foreignField': foreign_field,
                    'as': collection_name,
                    'pipeline': [{'$match': {'_id': ObjectId(lookup_id)}}],
                }
            },
            {
                '$unwind': {
                    'path': f'${collection_name}',
                    'preserveNullAndEmptyArrays': True,
                }
            },
        ]
    )
    models = []
    async for document in documents:
        models.append(parent_model.from_mongo(document))
    return models[0]


async def create_one(db, collection_name: str, id: str, data, foreign_field: str, local_field: str, parent_collection_name: str, parent_model: MongoModel) -> MongoModel:
    """
    Create one document in the database with a lookup
    """
    print(data)
    response = await db[collection_name].insert_one(data.mongo())
    return await get_one(db, collection_name, id, response.inserted_id, foreign_field, local_field, parent_collection_name, parent_model)


async def replace_one(db, collection_name: str, id: str, lookup_id: str, data, foreign_field: str, local_field: str, parent_collection_name: str, parent_model: MongoModel) -> MongoModel:
    """
    Update one document in the database with a lookup
    """
    response = await db[collection_name].replace_one({"_id": ObjectId(lookup_id)}, data.mongo())
    return await get_one(db, collection_name, id, lookup_id, foreign_field, local_field, parent_collection_name, parent_model)


async def update_one(db, collection_name: str, id: str, lookup_id: str, data, foreign_field: str, local_field: str, parent_collection_name: str, parent_model: MongoModel) -> MongoModel:
    """
    Update one document in the database with a lookup
    """
    response = await db[collection_name].update_one({"_id": ObjectId(lookup_id)}, {"$set": data.mongo()})
    return await get_one(db, collection_name, id, lookup_id, foreign_field, local_field, parent_collection_name, parent_model)


async def delete_one(db, collection_name: str, id: str, lookup_id: str, foreign_field: str, local_field: str, parent_collection_name: str, parent_model: MongoModel) -> MongoModel:
    """
    Delete one document in the database with a lookup
    """
    response = await db[collection_name].delete_one({"_id": ObjectId(lookup_id)})
    return await get_one(db, collection_name, id, lookup_id, foreign_field, local_field, parent_collection_name, parent_model)



from .core import (
    CRUDRouter,
    MongoModel,
    MongoObjectId,
    CRUDLookup,
)

__all__ = [
    "CRUDRouter",
    "MongoModel",
    "MongoObjectId",
    "CRUDLookup",
]

__version__ = "0.0.4"
__author__ = "Pierre DUVEAU"
__credits__ = ["Pierre DUVEAU", "Adam Watkins"]



from bson import ObjectId
from pydantic import BaseModel


class MongoModel(BaseModel):

    class Config:
        allow_population_by_field_name = True
        json_encoders = {ObjectId: str}

    @classmethod
    def from_mongo(cls, data: dict):
        """We must convert _id into "id". """
        if not data:
            return data
        mid = data.pop('_id', None)
        return cls(**dict(data, id=mid))

    def mongo(self, add_id: bool = False, **kwargs):
        exclude_none = kwargs.pop('exclude_none', True)
        by_alias = kwargs.pop('by_alias', True)

        parsed = self.dict(
            exclude_none=exclude_none,
            by_alias=by_alias,
            **kwargs,
        )

        # Mongo uses `_id` as default key. We should stick to that as well.
        if '_id' not in parsed and 'id' in parsed:
            parsed['_id'] = parsed.pop('id')
        if '_id' not in parsed and 'id' not in parsed and add_id:
            parsed['_id'] = ObjectId()
        return parsed

    def __init__(self, **pydict):
        super().__init__(**pydict)
        if pydict.get('_id'):
            self.id = pydict.pop('_id')



from bson import ObjectId


class MongoObjectId(ObjectId):
    @classmethod
    def __get_validators__(cls):
        yield cls.validate

    @classmethod
    def validate(cls, v):
        if not ObjectId.is_valid(v):
            raise ValueError("Invalid objectid")
        return ObjectId(v)

    @classmethod
    def __modify_schema__(cls, field_schema):
        field_schema.update(type="string")



from bson import ObjectId

from ...models.mongo_model import MongoModel


async def get_all(db, id: str, parent_collection_name: str, embed_name: str, model: MongoModel) -> list:
    """
    Get all embeded documents from the database
    """
    try:
        documents = db[parent_collection_name].aggregate([
            {
                '$match': {
                    '_id': ObjectId(id)
                }
            },
            {
                '$unwind': f'${embed_name}'
            },
            {
                '$replaceRoot': {
                    'newRoot': f"${embed_name}"
                }
            }
        ])

        models = []
        async for document in documents:
            models.append(model.from_mongo(document))
        return models
    except Exception as e:
        return []


async def get_one(db, id: str, embed_id: str, parent_collection_name: str, embed_name: str, model: MongoModel) -> MongoModel:
    """
    Get a document from the database
    """
    try:
        document = db[parent_collection_name].aggregate([
            {
                '$match': {
                    '_id': ObjectId(id)
                }
            },
            {
                '$unwind': f'${embed_name}'
            },
            {
                '$replaceRoot': {
                    'newRoot': f"${embed_name}"
                }
            }, {
                '$match': {
                    '_id': ObjectId(embed_id)
                }
            }
        ])
        models = []
        async for document in document:
            models.append(model.from_mongo(document))
        return models[0]
    except Exception as e:
        return None


async def create_one(db, id: str, parent_collection_name: str, embed_name: str, data: MongoModel, model: MongoModel) -> MongoModel:
    """
    Create a new document in the database
    """
    document_mongo = data.mongo(add_id=True)
    document = await db[parent_collection_name].update_one(
        {'_id': ObjectId(id)},
        {'$push': {embed_name: document_mongo}}
    )

    return model.from_mongo(document_mongo)


async def update_one(db, id: str, embed_id: str, parent_collection_name: str, embed_name: str, data: MongoModel, model: MongoModel) -> MongoModel:
    """
    Update a document in the database
    """
    document_mongo = data.mongo(add_id=False)
    document = await db[parent_collection_name].update_one(
        {'_id': ObjectId(id), f'{embed_name}._id': ObjectId(embed_id)},
        {'$set': {f'{embed_name}.$': document_mongo}}
    )

    return model.from_mongo(document_mongo)


async def delete_one(db, id: str, embed_id: str, parent_collection_name: str, embed_name: str, model: MongoModel) -> MongoModel:
    """
    Delete a document in the database
    """
    try:
        document = await db[parent_collection_name].update_one(
            {'_id': ObjectId(id)},
            {'$pull': {f'{embed_name}': {'_id': ObjectId(embed_id)}}}
        )
        return {'deleted': True} if await get_one(db, id, embed_id, parent_collection_name, embed_name, model) is None else None
    except Exception as e:
        return None


from typing import Any, Callable
from fastapi import APIRouter

from ..CRUDRouterFactory import CRUDRouterFactory
from ...models.CRUDLookup import CRUDLookup

class CRUDLookupRouterFactory(APIRouter):

    def __init__(self, parent_router: CRUDRouterFactory, child_args: CRUDLookup, *args, **kwargs):
        self.parent_router = parent_router
        self.child_args = child_args

    def _add_api_route(
            self,
            path: str,
            endpoint: Callable,
            **kwargs: Any) -> None:
        self.parent_router._add_api_route(path, endpoint, **kwargs)

    def _get_all(self, *args: Any, **kwargs: Any) -> Callable[..., Any]:
        return self.parent_router._get_all(*args, **kwargs)
    
    def _get_one(self, *args: Any, **kwargs: Any) -> Callable[..., Any]:
        return self.parent_router._get_one(*args, **kwargs)
    
    def _create_one(self, *args: Any, **kwargs: Any) -> Callable[..., Any]:
        return self.parent_router._create_one(*args, **kwargs)

    def _replace_one(self, *args: Any, **kwargs: Any) -> Callable[..., Any]:
        return self.parent_router._update_one(*args, **kwargs)
    
    def _update_one(self, *args: Any, **kwargs: Any) -> Callable[..., Any]:
        return self.parent_router._delete_one(*args, **kwargs)
    
    def _delete_one(self, *args: Any, **kwargs: Any) -> Callable[..., Any]:
        return self.parent_router._delete_one(*args, **kwargs)
    


from typing import Any, Callable, List

from fastapi import HTTPException
from .CRUDRouterFactory import CRUDRouterFactory
from . import CRUDRouterRepository
from .embed.CRUDEmbedRouter import CRUDEmbedRouter
from .lookup.CRUDLookupRouter import CRUDLookupRouter
from ..models.CRUDEmbed import CRUDEmbed
from ..models.CRUDLookup import CRUDLookup
from ..models.mongo_model import MongoModel


class CRUDRouter(CRUDRouterFactory):
    """
    CRUDRouter is a class that extends CRUDRouterFactory and implements the CRUD operations for a given model.

    :param model: The model to be used for the CRUD operations.
    :type model: MongoModel
    :param db: The database to be used for the CRUD operations.
    :type db: Database
    :param collection_name: The name of the collection to be used for the CRUD operations.
    :type collection_name: str
    :param lookups: A list of lookup objects to be used for the CRUD operations.
    :type lookups: List[CRUDLookup]
    :param args: The args to be passed to the CRUDRouterFactory.
    :type args: Any
    :param kwargs: The kwargs to be passed to the CRUDRouterFactory.
    :type kwargs: Any
    """
    def __init__(self, model, db, collection_name, lookups: List[CRUDLookup] = [], *args, **kwargs) -> None:
        super().__init__(model, db, collection_name, *args, **kwargs)
        self._register_routes()
        try:
            for lookup in lookups:
                CRUDLookupRouter(self, lookup, *args, **kwargs)
            for key, value in self.model.__fields__.items():
                try:
                    if(key in self.model.__fields__ and issubclass(self.model.__fields__[key].type_, MongoModel)):
                        embed_model = CRUDEmbed(self.model.__fields__[key].type_, key)
                        CRUDEmbedRouter(self, embed_model, *args, **kwargs)
                except Exception as e:
                    continue

        except Exception as e:
            print(e)

    def _get_all(self, *args: Any, **kwargs: Any) -> Callable[..., Any]:
        async def route() -> list:
            response = await CRUDRouterRepository.get_all(self.db, self.model, self.collection_name)
            if (not len(response)):
                raise HTTPException(404, "Empty collection")
            return response
        return route

    def _get_one(self, *args: Any, **kwargs: Any) -> Callable[..., Any]:
        async def route(id: str) -> self.model:
            response = await CRUDRouterRepository.get_one(self.db, self.model, self.collection_name, id)
            if (response is None):
                raise HTTPException(404, "Document not found")
            return response
        return route

    def _create_one(self, *args: Any, **kwargs: Any) -> Callable[..., Any]:
        async def route(data: self.model) -> self.model:
            response = await CRUDRouterRepository.create_one(self.db, self.model, self.collection_name, data)
            if (response is None):
                raise HTTPException(422, "Document not created")
            return response
        return route

    def _replace_one(self, *args: Any, **kwargs: Any) -> Callable[..., Any]:
        async def route(id: str, data: self.model) -> self.model:
            response = await CRUDRouterRepository.replace_one(self.db, self.model, self.collection_name, id, data)
            if (response is None):
                raise HTTPException(422, "Document not replaced")
            return response
        return route

    def _update_one(self, *args: Any, **kwargs: Any) -> Callable[..., Any]:
        async def route(id: str, data: self.model) -> self.model:
            response = await CRUDRouterRepository.update_one(self.db, self.model, self.collection_name, id, data)
            if (response is None):
                raise HTTPException(422, "Document not updated")
            return response
        return route

    def _delete_one(self, *args: Any, **kwargs: Any) -> Callable[..., Any]:
        async def route(id: str) -> self.model:
            response = await CRUDRouterRepository.delete_one(self.db, self.collection_name, id)
            if (response is None):
                raise HTTPException(422, "Document not deleted")
            return response
        return route

    def _register_routes(self):
        self._add_api_route(
            "/",
            self._get_all(),
            response_model=List[self.model],
            methods=["GET"],
            summary=f"Get All {self.model.__name__} from the collection",
            description=f"Get All {self.model.__name__} from the collection")
        self._add_api_route(
            "/{id}",
            self._get_one(),
            response_model=self.model,
            methods=["GET"],
            summary=f"Get One {self.model.__name__} by {{id}} from the collection",
            description=f"Get One {self.model.__name__} by {{id}} from the collection",
        )
        self._add_api_route("/", self._create_one(),
                            response_model=self.model, methods=["POST"],
                            summary=f"Create One {self.model.__name__} in the collection",
                            description=f"Create One {self.model.__name__} in the collection")
        self._add_api_route(
            "/{id}",
            self._update_one(),
            response_model=self.model,
            methods=["PATCH"],
            summary=f"Update One {self.model.__name__} by {{id}} in the collection",
            description=f"Update One {self.model.__name__} by {{id}} in the collection",
        )
        self._add_api_route(
            "/{id}",
            self._replace_one(),
            response_model=self.model,
            methods=["PUT"],
            summary=f"Replace One {self.model.__name__} by {{id}} in the collection",
            description=f"Replace One {self.model.__name__} by {{id}} in the collection",
        )
        self._add_api_route(
            "/{id}",
            self._delete_one(),
            response_model=self.model,
            methods=["DELETE"],
            summary=f"Delete One {self.model.__name__} by {{id}} from the collection",
            description=f"Delete One {self.model.__name__} by {{id}} from the collection",
        )



from bson import ObjectId
from ..models.mongo_model import MongoModel


async def get_all(db, model: MongoModel, collection_name) -> list:
    """
    Get all documents from the database
    """
    documents = []
    async for document in db[collection_name].find():
        documents.append(model.from_mongo(document))
    return documents


async def get_one(db, model: MongoModel, collection_name: str, id: str):
    """
    Get one document from the database
    """
    response = await db[collection_name].find_one({"_id": ObjectId(id)})
    return model.from_mongo(response) if response is not None else None


async def create_one(db, model: MongoModel, collection_name, data: MongoModel):
    """
    Create one document in the database
    """
    response = await db[collection_name].insert_one(data.mongo())
    response = await db[collection_name].find_one({"_id": response.inserted_id})
    return model.from_mongo(response)


async def replace_one(db, model: MongoModel, collection_name, id: str, data: MongoModel):
    """
    Update one document in the database
    """
    response = await db[collection_name].replace_one({"_id": ObjectId(id)}, data.mongo())
    response = await db[collection_name].find_one({"_id": response.upserted_id if response.upserted_id is not None else ObjectId(id)})
    return model.from_mongo(response)


async def update_one(db, model: MongoModel, collection_name, id: str, data: MongoModel):
    """
    Update one document in the database
    """
    response = await db[collection_name].update_one({"_id": ObjectId(id)}, {"$set": data.mongo()})
    response = await db[collection_name].find_one({"_id": response.upserted_id if response.upserted_id is not None else ObjectId(id)})
    return model.from_mongo(response)


async def delete_one(db, collection_name, id):
    """
    Delete one document from the database
    """
    await db[collection_name].delete_one({"_id": ObjectId(id)})
    return {"id": id}



from .CRUDRouter import CRUDRouter
from .embed.CRUDEmbedRouter import CRUDEmbedRouter
from .lookup.CRUDLookupRouter import CRUDLookupRouter

__all__ = [
    "CRUDRouter",
    "CRUDEmbedRouter",
    "CRUDLookupRouter",
]



from typing import Any, Callable, List
from fastapi import HTTPException

from ...models.CRUDEmbed import CRUDEmbed

from .CRUDEmbedRouterFactory import CRUDEmbedRouterFactory
from . import CRUDEmbedRouterRepository

class CRUDEmbedRouter(CRUDEmbedRouterFactory):

    def __init__(self, parent_router, child_args: CRUDEmbed, *args, **kwargs) -> None :
        self.prefix = "/{id}/" + child_args.embed_name
        self.db = parent_router.db
        self.model = child_args.model
        self.embed_name = child_args.embed_name
        super().__init__(parent_router, child_args, *args, **kwargs)
        self._register_routes()

    def _get_all(self, *args: Any, **kwargs: Any) -> Callable[..., Any]:
        async def route(id: str) -> self.model:
            response = await CRUDEmbedRouterRepository.get_all(self.db, id, self.parent_router.collection_name, self.embed_name, self.model)
            if (not len(response)):
                raise HTTPException(404, "Empty collection")
            return response
        return route
    
    def _get_one(self, *args: Any, **kwargs: Any) -> Callable[..., Any]:
        async def route(id: str, embed_id: str) -> self.model:
            response = await CRUDEmbedRouterRepository.get_one(self.db, id, embed_id, self.parent_router.collection_name, self.embed_name, self.model)
            if (response is None):
                raise HTTPException(404, "Document not found")
            return response
        return route
    
    def _create_one(self, *args: Any, **kwargs: Any) -> Callable[..., Any]:
        async def route(id: str, data: self.model) -> self.model:
            response = await CRUDEmbedRouterRepository.create_one(self.db, id, self.parent_router.collection_name, self.embed_name, data, self.model)
            if (response is None):
                raise HTTPException(422, "Document not created")
            return response
        return route
        
    def _update_one(self, *args: Any, **kwargs: Any) -> Callable[..., Any]:
        async def route(id: str, embed_id: str, data: self.model) -> self.model:
            response = await CRUDEmbedRouterRepository.update_one(self.db, id, embed_id, self.parent_router.collection_name, self.embed_name, data, self.model)
            if (response is None):
                raise HTTPException(422, "Document not updated")
            return response
        return route
    
    def _delete_one(self, *args: Any, **kwargs: Any) -> Callable[..., Any]:
        async def route(id: str, embed_id: str) -> self.model:
            response = await CRUDEmbedRouterRepository.delete_one(self.db, id, embed_id, self.parent_router.collection_name, self.embed_name, self.model)
            if (response is None):
                raise HTTPException(422, "Document not deleted")
            return response
        return route
    
    def _register_routes(self) -> None:
        self._add_api_route(
            path=self.prefix,
            endpoint=self._get_all(),
            methods=["GET"],
            response_model=List[self.model],
            tags=[self.embed_name],
            summary=f"Get All {self.model.__name__} embedded into a {self.parent_router.model.__name__}",
            description=f"Get All {self.model.__name__} embedded into a {self.parent_router.model.__name__}"
        )
        self._add_api_route(
            path=self.prefix + "/{embed_id}",
            endpoint=self._get_one(),
            methods=["GET"],
            response_model=self.model,
            tags=[self.embed_name],
            summary=f"Get One {self.model.__name__} embedded into a {self.parent_router.model.__name__}",
            description=f"Get One {self.model.__name__} embedded into a {self.parent_router.model.__name__}"
        )
        self._add_api_route(
            path=self.prefix,
            endpoint=self._create_one(),
            methods=["POST"],
            response_model=self.model,
            tags=[self.embed_name],
            summary=f"Create One {self.model.__name__} embedded into a {self.parent_router.model.__name__}",
            description=f"Create One {self.model.__name__} embedded into a {self.parent_router.model.__name__}"

        )
        self._add_api_route(
            path=self.prefix + "/{embed_id}",
            endpoint=self._update_one(),
            methods=["PATCH"],
            response_model=self.model,
            tags=[self.embed_name],
            summary=f"Update One {self.model.__name__} embedded into a {self.parent_router.model.__name__}",
            description=f"Update One {self.model.__name__} embedded into a {self.parent_router.model.__name__}"
        )
        self._add_api_route(
            path=self.prefix + "/{embed_id}",
            endpoint=self._delete_one(),
            methods=["DELETE"],
            response_model=Any,
            tags=[self.embed_name],
            summary=f"Delete One {self.model.__name__} embedded into a {self.parent_router.model.__name__}",
            description=f"Delete One {self.model.__name__} embedded into a {self.parent_router.model.__name__}"
        )


from setuptools import setup, find_packages



with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="fastapi-crudrouter-mongodb",
    version="0.0.4",
    author="Pierre DUVEAU",
    author_email="",
    description="A dynamic FastAPI router that automatically creates CRUD routes for your mongodb models",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/pierrod/fastapi-crudrouter-mongodb",
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        "fastapi",
        "pydantic",
        "pymongo",
        "motor"
    ],
    classifiers=[
        "Operating System :: OS Independent",
        "Development Status :: 3 - Alpha",
        "Topic :: Internet",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: HTTP Servers",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Code Generators",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    python_requires=">=3.7",
)




from abc import abstractmethod
from typing import Any, Callable
from fastapi import APIRouter, HTTPException


class CRUDRouterFactory(APIRouter):

    def __init__(self, model, db, collection_name, *args, **kwargs):
        self.model = model
        self.collection_name = collection_name
        self.db = db
        super().__init__(*args, **kwargs)

    def _add_api_route(
            self,
            path: str,
            endpoint: Callable,
            **kwargs: Any) -> None:
        super().add_api_route(path, endpoint, **kwargs)

    @abstractmethod
    def _get_all(self, *args: Any, **kwargs: Any) -> Callable[..., Any]:
        raise NotImplementedError

    @abstractmethod
    def _get_one(self, *args: Any, **kwargs: Any) -> Callable[..., Any]:
        raise NotImplementedError

    @abstractmethod
    def _create_one(self, *args: Any, **kwargs: Any) -> Callable[..., Any]:
        raise NotImplementedError

    @abstractmethod
    def _replace_one(self, *args: Any, **kwargs: Any) -> Callable[..., Any]:
        raise NotImplementedError

    @abstractmethod
    def _update_one(self, *args: Any, **kwargs: Any) -> Callable[..., Any]:
        raise NotImplementedError

    @abstractmethod
    def _delete_one(self, *args: Any, **kwargs: Any) -> Callable[..., Any]:
        raise NotImplementedError

    def _raise(self, e: Exception, status_code: int = 422) -> HTTPException:
        raise HTTPException(422, ", ".join(e.args)) from e
